﻿
using System.Diagnostics;
using System.Linq;
using System.Collections.Generic;
using System.Collections;
using System;
using System.Xml.Linq;

namespace MONOW_PAL_PARSER
{
    public class ClsDataPayloadPalSensors : ITweDataPayload
    {
        private bool _bStored;
        private ClsDataPalSenseElement[] _aElements;

        public ClsDataPalSenseElement[] Elements
        {
            get
            {
                return _aElements;
            }
            set
            {
                _aElements = value;
            }
        }

        /// <summary>
    /// Sensor Code による要素の抽出を行う
    /// </summary>
    /// <param name="snscd"></param>
    /// <returns>リスト形式の要素</returns>
        public List<ClsDataPalSenseElement> this[uint snscd]
        {
            get
            {
                var objs = new List<ClsDataPalSenseElement>();

                foreach (ClsDataPalSenseElement e in _aElements)
                {
                    if (e.SensorCode == snscd)
                        objs.Add(e);
                }

                return objs;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="snscd"></param>
        /// <returns></returns>
        public List<ClsDataPalSenseElement> SensorCode(uint snscd)
        {
            var objs = new List<ClsDataPalSenseElement>();

            foreach (ClsDataPalSenseElement e in _aElements)
            {
                if (e.SensorCode == snscd)
                    objs.Add(e);
            }

            return objs;
        }


        public bool IsStored
        {
            get
            {
                return _bStored;
            }
        }

        public byte[] ToByteArray()
        {
            return null;
        }

        public bool ParseByteArray(ref byte[] b)
        {
            bool _ret;
            int i;
            int j;
            int bIdx;
            int iLen = b[0]; // number of sensor data elements
            Elements = new ClsDataPalSenseElement[iLen];

            try
            {
                bIdx = 1;
                var loopTo = iLen - 1;
                for (i = 0; i <= loopTo; i++)
                {
                    ClsDataPalSenseElement e = new ClsDataPalSenseElement();

                    byte bDataType = b[bIdx];
                    e.SensorCode = b[bIdx + 1];
                    e.Ex = null;

                    // Extra byte,
                    //   - even if 0x10 bit is not set, b[2] is present with 0 value.
                    if ((bDataType & 0x10) == 0x10)
                    {
                        e.Ex = new byte();
                        e.Ex = b[bIdx + 2];
                    }
                    byte bLen = b[bIdx + 3];

                    bIdx += 4;

                    if ((bDataType & 0x80) == 0x80)
                    {
                        // Sensor Error
                        e.IsError = true;
                        e.ErrorCode = (byte)(bDataType & 0x7F);
                    }
                    else
                    {
                        // Sensor is valid
                        e.IsError = false;

                        int DatType = bDataType & 0x7;
                        int iDataCt;

                        switch (DatType)
                        {
                            case 0: // uchar
                                {
                                    iDataCt = bLen;
                                    byte[] a =new byte[iDataCt];
                                    var loopTo1 = iDataCt;
                                    for (j = 0; j < loopTo1; j++)
                                        a[j] = b[bIdx + j];
                                    e.Data = a;
                                    break;
                                }

                            case 4: // ichar
                                {
                                    iDataCt = bLen;
                                    sbyte[] a =new sbyte[iDataCt];
                                    var loopTo2 = iDataCt;
                                    for (j = 0; j < loopTo2; j++)
                                        a[j] = (sbyte)b[bIdx + j];
                                    e.Data = a;
                                    break;
                                }

                            case 1: // ushort
                                {
                                    iDataCt = bLen / 2;
                                    ushort[] a = new ushort[iDataCt];
                                    var loopTo3 = iDataCt;
                                    for (j = 0; j < loopTo3; j++)
                                        a[j] = PAL_Utils.get_BE_WORD(ref b, bIdx + j * 2);
                                    e.Data = a;
                                    break;
                                }

                            case 5: // short
                                {
                                    iDataCt = bLen / 2;
                                    short[] a = new short[iDataCt];
                                    var loopTo4 = iDataCt;
                                    for (j = 0; j < loopTo4; j++)
                                    {
                                        a[j] = (short)PAL_Utils.get_BE_WORD(ref b, bIdx + j * 2);
                                    }
                                    e.Data = a;
                                    break;
                                }

                            case 2: // uint
                                {
                                    iDataCt = bLen / 4;
                                    uint[] a = new uint[iDataCt];
                                    var loopTo5 = iDataCt;
                                    for (j = 0; j < loopTo5; j++)
                                        a[j] = PAL_Utils.get_BE_DWORD(ref b, bIdx + j * 4);
                                    e.Data = a;
                                    break;
                                }

                            case 6: // short
                                {
                                    iDataCt = bLen / 4;
                                    int[] a = new int[iDataCt];
                                    var loopTo6 = iDataCt;
                                    for (j = 0; j < loopTo6; j++)
                                    {
                                        a[j] = (int)PAL_Utils.get_BE_DWORD(ref b, bIdx + j * 4);
                                    }
                                    e.Data = a;
                                    break;
                                }

                            case 7: // 可変長
                                {
                                    iDataCt = bLen - 1;
                                    byte[] a = new byte[bLen];
                                    var loopTo7 = iDataCt;
                                    for (j = 0; j <= loopTo7; j++)
                                        a[j] = b[bIdx + j];
                                    e.Data = a;
                                    break;
                                }
                        }
                    }

                    Elements[i] = e;

                    bIdx += bLen;
                }

                _bStored = true;
                _ret = true;
            }
            catch (Exception ex)
            {
                _bStored = false;
                _ret = false;
            }
            return _ret;
        }

        public byte GetCmdId()
        {
            return 0xFF;
        }

        public ClsDataPayloadPalSensors()
        {
            clearData();
        }

        public ClsDataPayloadPalSensors(ref byte[] _b)
        {
            ParseByteArray(ref _b);

            if (!IsStored)
                throw new Exception();
        }

        private void clearData()
        {
            _bStored = false;
        }
    }
}
